using System;
using System.Collections;

namespace PageBoxGrid
{
	/// <summary>
	/// <p>Part of the Coordinator set. Heap management.</p>
	/// <p>Copyright (c) 2002 Alexis Grandemange<br/>
	/// Mail: alexis.grandemange@pagebox.net</p>
	/// <pre>This program is free software; you can redistribute it and/or
	/// modify it under the terms of the GNU Lesser General Public
	/// License as published by the Free Software Foundation; version
	/// 2.1 of the License.
	/// This library is distributed in the hope that it will be useful,
	/// but WITHOUT ANY WARRANTY; without even the implied warranty of
	/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	/// GNU Lesser General Public License for more details.
	/// A copy of the GNU Lesser General Public License lesser.txt should be
	/// included in the distribution.</pre>
	/// </summary>
	public class Heap
	{
		/// <summary>
		/// Key: Web server session ID
		/// Value: Hastable whose key is the ID of the object source and value is the stored object
		/// </summary>
		private static Hashtable heap = new Hashtable();

		/// <summary>
		/// Put an object on the Heap
		/// </summary>
		/// <param name="sourceID">ID of the object source</param>
		/// <param name="sessionID">Web server session ID</param>
		/// <returns>Object found in the Heap or null</returns>
		public static object Get(string sourceID, string sessionID) 
		{
			lock(typeof(Heap)) 
			{
				if (!heap.Contains(sessionID))
					return null;
				Hashtable ht = (Hashtable)heap[sessionID];
				if (!ht.Contains(sourceID))
					return null;
				return ht[sourceID];
			}
		}

		/// <summary>
		/// Retrieve an object from the Heap
		/// </summary>
		/// <param name="sourceID">ID of the object source</param>
		/// <param name="sessionID">Web server session ID</param>
		/// <param name="toStore">Object to store in the Heap</param>
		public static void Put(string sourceID, string sessionID, object toStore) 
		{
			lock(typeof(Heap)) 
			{
				Hashtable ht;
				bool newSession = false;
				if (!heap.Contains(sessionID)) 
				{
					ht = new Hashtable();
					newSession = true;
				}
				else
					ht = (Hashtable)heap[sessionID];
				if (!ht.Contains(sourceID))
					ht.Add(sourceID, toStore);
				else
					ht[sourceID] = toStore;
				if (newSession)
					heap.Add(sessionID, ht);
			}
		}

		/// <summary>
		/// Remove an object from the Heap
		/// </summary>
		/// <param name="sourceID">ID of the object source</param>
		/// <param name="sessionID">Web server session ID</param>
		public static void Remove(string sourceID, string sessionID) 
		{
			lock(typeof(Heap)) 
			{
				if (heap.Contains(sessionID)) 
				{
					Hashtable ht = (Hashtable)heap[sessionID];
					if (ht.Contains(sourceID))
						ht.Remove(sourceID);
					if (ht.Count == 0)
						heap.Remove(sessionID);
				}
			}
		}
	}
}
